%*************************************************************************
% Matlab interface for MTracker robot
% (c) KSIS, D. Pazderski 2015
% Edit: K. akomy 2017
%*************************************************************************
close all;
clc

Tf = 10;            % Simulatuion time [s]
Ts = 0.05;          % Sampling time [s]
COM = 1;            % COM port number
tic;

if (MTrackerDriver('open', COM) == -1)
    return;
end

n = floor(Tf/Ts)+1; % Number of samples

% Initialization of buffers to store data
t = zeros(1, n);
dataReady = zeros(1, n);
w = zeros(2, n);
wd = zeros(2, n);
ud = zeros(2, n);
ur = zeros(2, n);
q = zeros(3, n);
qr = zeros(3, n);

% Define user data


% Initial localization
q_i = TrajectoryGenerator(0);

i = 0; tau = 0;
w_i = [0; 0];
% Main control loop

figure
hold on;
grid on;
axis([-0.6 0.6 -0.6 0.6]);
axis equal;
h_old = plot(0,0,0,0,'r','LineWidth',1.5);
legend('q','q_{ref}');
title('Real and reference odometry');
xlabel('x[m]');
ylabel('y[m]');


MTrackerDriver('setOdometry', q_i); 
wait(tau, 0.1);
data = MTrackerDriver('read');
tic;

disp('Robot is started.');
while (tau < Tf)
    
    i = i+1;
    if(i==1)
        tic;
    end
    tau = toc;

    % Compute control law
    [ud_i, qr_i, ur_i] = Controller(q_i, tau);
    wd_i = ComputeWheelsVelocities(ud_i);
%     wd_i = ReferenceWheelVelocityGenerator(t);
          
%     wd_i = [10;10];
            
    % Communication with the robot
    MTrackerDriver('sendVelocity', wd_i); 
    wait(tau, Ts);
    data = MTrackerDriver('read');
    
    % Check if new data is available
    if (data(1) == 1 && i>1)
        q_i = data(2:4)';
        w_i = data(5:6)';
    end
    
    dataReady(:,i) = data(1);
    q(:,i) = q_i;
    qr(:,i) = qr_i;
    w(:,i) = w_i; 
    wd(:,i) = wd_i;
    ud(:,i) = ud_i;
    ur(:,i) = ur_i;
    t(i) = tau;  
    
    h = plot(q(1,1:i), q(2,1:i),qr(1,1:i),qr(2,1:i),'ro','LineWidth',1.5);
    delete(h_old);
    h_old = h;
    drawnow;
end    

MTrackerDriver('close');

% Adjust buffers
t = t(:, 1:i);
w = w(:, 1:i);
wd = wd(:, 1:i);
ud = ud(:, 1:i);
ur = ur(:, 1:i);
qr = qr(:, 1:i);
q = q(:, 1:i);

disp('Robot is stopped.');